'##############################################################################################################################
' Simple example terminal programm for the USB chip FT232AM using the driver DLL FTD2XX.DLL
' Copyright:             (c) 2003 Roland Walter, support@rowalt.de, www.rowalt.de/mc/
' Programming language:  PowerBasic for Windows 7.0 (see www.powerbasic.com)
' Last change:           05.02.2003 (DD.MM.YYYY)
' Legal state:           Freeware
'
' Comments:
' This program was published along with an article in the german journal FUNKAMATEUR 03/2003.
' The used hardware was a small FT232AM+EEPROM board with plugs for the TTL signal lines to connect an AVR microcontroller.
' For basic tests You can simply use a small wire to connect TxD with RxD (this echos the data You sent). The same You can
' do with the different input/output Handshake lines (or You might like to use use a LowCurrent LED with resistor connected
' to Gnd to check the state of the output lines and a 220 Ohm resistor connected to Gnd to set the input lines).
'##############################################################################################################################
#Register All
#Dim All
#Compile Exe "UsbTerm.exe"
#Option Version3            'NT3/Win95 as a minimum

#Include "WIN32API.INC"
#Include "FTD2XX.INC"       'Declarations for the FTD2XX.DLL
#Resource "USBTERM.PBR"     'Resource file containing the dialog template (created from USBTERM.RES using the tool PBRES.EXE)

'Flags for FT_GetModemStatus (own flags added to FTD2XX.INC which are not declared in FTD2XX.H)
'%FT_STATUS_CTS  = &H00000010???   '=&B00010000
'%FT_STATUS_DSR  = &H00000020???   '=&B00100000
'%FT_STATUS_RING = &H00000040???   '=&B01000000
'%FT_STATUS_DCD  = &H00000080???   '=&B10000000

'Dialog control IDs:
%IDD_DLGMAIN    =1          'Resource ID of the main dialog (the one and only we have)
%IDC_LOGLIST    =400        'Log listbox
%IDC_TEXTTOSEND =100        'EDIT control with the text to send
%IDC_SENDTEXT   =1          'Button "Send"
%IDC_EXIT       =2          'Exit program; not a control, but ID 2 acts as "Cancel" (ESC key)
%IDC_RTS_OUT    =201        'Handshake output line checkbox for RTS
%IDC_DTR_OUT    =202        'Handshake output line checkbox for DTR
%IDC_CTS_IN     =301        'Handshake input line checkbox for CTS
%IDC_DSR_IN     =302        'Handshake input line checkbox for DSR
%IDC_DCD_IN     =303        'Handshake input line checkbox for DCD
%IDC_RING_IN    =304        'Handshake input line checkbox for Ring (Ri)

Declare Sub AddListItem(ByVal hwndList As Long,szItemText As Asciiz)
'******************************************************************************************************************************
Function WinMain(ByVal hInst As Long,ByVal hPrevInst As Long,lpCmdLine As Asciiz Ptr,ByVal iCmdShow As Long) As Long
  DialogBox hInst,ByVal %IDD_DLGMAIN,0,CodePtr(DlgMainProc)
  Function=0
End Function
'******************************************************************************************************************************
Function DlgMainProc(ByVal hDlg As Long,ByVal wMsg As Long,ByVal wParam As Long,ByVal lParam As Long) As Long
  Static hwndList As Long,szBuffer As Asciiz*255
  Static hUsbDevice As Dword,dwBytesDone As Dword,ftStatus As Dword,dwModemStatus As Dword
  '
  Select Case wMsg
  Case %WM_INITDIALOG
    hwndList=GetDlgItem(hDlg,%IDC_LOGLIST)         'Get the handle of the "Log" listbox
    SendDlgItemMessage hDlg,%IDC_TEXTTOSEND,%EM_LIMITTEXT,SizeOf(szBuffer),0
    '
    If FT_Open(0,hUsbDevice)=%FT_OK Then 'Open device, get handle
      AddListItem hwndList,"Open USB device OK"
    Else
      AddListItem hwndList,"Open USB device failed"
    End If
    If FT_SetBaudRate(hUsbDevice,9600)=%FT_OK Then  'Set baud rate to 9600
      AddListItem hwndList,"SetBaudRate 9600 OK"
    Else
      AddListItem hwndList,"SetBaudRate failed"
    End If
    If FT_SetDataCharacteristics(hUsbDevice,%FT_BITS_8,%FT_STOP_BITS_1,%FT_PARITY_NONE)=%FT_OK Then '8 data bits, 1 stop bit, no parity
      AddListItem hwndList,"SetDataCharacteristics 8N1 OK"
    Else
      AddListItem hwndList,"SetDataCharacteristics failed"
    End If
    If FT_SetFlowControl(hUsbDevice,%FT_FLOW_NONE,0,0)=%FT_OK Then 'No flow control
      AddListItem hwndList,"SetFlowControl NONE OK"
    Else
      AddListItem hwndList,"SetFlowControl failed"
    End If
    If FT_SetTimeouts(hUsbDevice,2,0)=%FT_OK Then    'Set 1 milliseconds read timeout (the least possible value)
      AddListItem hwndList,"SetTimeouts OK"
    Else
      AddListItem hwndList,"SetTimeouts failed"
    End If
    '
    'To read data from USB, normally the best way is to use an own Thread. But for simplicity we use a timer instead.
    'After each WM_TIMER message we read out the USB input queue.
    SetTimer hDlg,1,200,ByVal 0    'Timer ID 1, results WM_TIMER message each 200 milliseconds (this is enough for us)
    '
    Function=%TRUE                 'Let Windows set the focus to the default control
  Case %WM_COMMAND
    Select Case LoWrd(wParam)
    Case %IDC_SENDTEXT          'Send the string which is in EDIT control
      GetDlgItemText hDlg,%IDC_TEXTTOSEND,szBuffer,SizeOf(szBuffer)
      If Len(szBuffer)=0 Then
        AddListItem hwndList,"Nothing to send"
      Else
        dwBytesDone=0
        If FT_Write(hUsbDevice,szBuffer,Len(szBuffer),dwBytesDone)=%FT_OK Then
          AddListItem hwndList,"Write OK,"+Str$(dwBytesDone)+" bytes written"
        Else
          AddListItem hwndList,"Write Failed"
        End If
      End If
    Case %IDC_RTS_OUT           'Handshake output line checkbox for RTS
      If IsDlgButtonChecked(hDlg,%IDC_RTS_OUT) Then
        If FT_SetRTS(hUsbDevice)=%FT_OK Then
          AddListItem hwndList,"RTS set OK"
        Else
          AddListItem hwndList,"RTS set failed"
        End If
      Else
        If FT_ClrRTS(hUsbDevice)=%FT_OK Then
          AddListItem hwndList,"RTS clear OK"
        Else
          AddListItem hwndList,"RTS clear failed"
        End If
      End If
    Case %IDC_DTR_OUT           'Handshake output line checkbox for DTR
      If IsDlgButtonChecked(hDlg,%IDC_DTR_OUT) Then
        If FT_SetDTR(hUsbDevice)=%FT_OK Then
          AddListItem hwndList,"DTR set OK"
        Else
          AddListItem hwndList,"DTR set failed"
        End If
      Else
        If FT_ClrDTR(hUsbDevice)=%FT_OK Then
          AddListItem hwndList,"DTR clear OK"
        Else
          AddListItem hwndList,"DTR clear failed"
        End If
      End If
    Case %IDC_EXIT              'ESC key
      EndDialog hDlg, 0          'Close dialog
    End Select  'SELECT CASE LOWRD(wParam)
    Function=0
  Case %WM_TIMER  'All 200 milliseconds: Read the USB input queue (and get line states)
    'First check whether there are characters waiting in the USB input queue.
    'If You like You can use FT_GetQueueStatus before FT_Read to check whether there are data in queue or not.
    'This speeds up the read-out process - don't forget that FT_Read waits depending on the timeout,
    'which is set with FT_SetTimeouts.
    ftStatus=FT_Read(hUsbDevice,szBuffer,SizeOf(szBuffer),dwBytesDone)
    If ftStatus=%FT_OK Then
      If dwBytesDone>0 Then
        AddListItem hwndList,"Data received: "+Left$(szBuffer,dwBytesDone)
      End If
    ElseIf ftStatus=%FT_IO_ERROR Then
        AddListItem hwndList,"I/O error while reading"
    End If
    '
    'Now retrieve the current state of the input Handshake signals:
    FT_GetModemStatus hUsbDevice,dwModemStatus
    If (dwModemStatus And %FT_STATUS_CTS)=%FT_STATUS_CTS Then     'CTS is high
      CheckDlgButton hDlg,%IDC_CTS_IN,%BST_CHECKED
    Else                                                          'CTS is low
      CheckDlgButton hDlg,%IDC_CTS_IN,%BST_UNCHECKED
    End If
    If (dwModemStatus And %FT_STATUS_DSR)=%FT_STATUS_DSR Then     'DSR is high
      CheckDlgButton hDlg,%IDC_DSR_IN,%BST_CHECKED
    Else                                                          'DSR is low
      CheckDlgButton hDlg,%IDC_DSR_IN,%BST_UNCHECKED
    End If
    If (dwModemStatus And %FT_STATUS_RING)=%FT_STATUS_RING Then   'Ri is high
      CheckDlgButton hDlg,%IDC_RING_IN,%BST_CHECKED
    Else                                                          'Ri is low
      CheckDlgButton hDlg,%IDC_RING_IN,%BST_UNCHECKED
    End If
    If (dwModemStatus And %FT_STATUS_DCD)=%FT_STATUS_DCD Then     'DCD is high
      CheckDlgButton hDlg,%IDC_DCD_IN,%BST_CHECKED
    Else                                                          'DCD is low
      CheckDlgButton hDlg,%IDC_DCD_IN,%BST_UNCHECKED
    End If
    'If dwModemStatus Then  'Debug code only
    '  AddListItem hwndList,"Modem status:"+Bin$(dwModemStatus,8)
    'End If
    Function=0
  Case %WM_CLOSE        'The dialog is about to close
    KillTimer hDlg,1    'Don't forget to kill the timer we set to read the USB input queue
    FT_Close hUsbDevice 'Close the USB device
    Function=0
  End Select            'SELECT CASE wMsg
End Function
'******************************************************************************************************************************
Sub AddListItem(ByVal hwndList As Long,szItemText As Asciiz)
  SendMessage hwndList,%LB_ADDSTRING,0,ByRef szItemText
  SendMessage hwndList,%LB_SETCURSEL,SendMessage(hwndList,%LB_GETCOUNT,0,0)-1,0
End Sub
